@@ -1,5 +1,6 @@
 var DSP_BIND = NewDnsProvider("bind", "BIND");
 var REG_CHANGEME = NewRegistrar("none");
+
 D("apex.com", REG_CHANGEME,
 	DnsProvider(DSP_BIND),
 	//SOA("@", "ns3.serverfault.com.", "sysadmin.stackoverflow.com.", 2020022300, 3600, 600, 604800, 1440),
@@ -9,5 +10,6 @@ D("apex.com", REG_CHANGEME,
 	//NAMESERVER("ns-cloud-c2.googledomains.com."),
 	// NOTE: CNAME at apex may require manual editing.
 	CNAME("@", "cnametest1.example.com."),
-	CNAME("www", "cnametest2.example.com.")
-)
+	CNAME("www", "cnametest2.example.com."),
+END);
+
