@@ -3,6 +3,7 @@ package models
 import (
 	"fmt"
 	"strings"
+	"sync"
 
 	"github.com/qdm12/reprint"
 	"golang.org/x/net/idna"
@@ -23,15 +24,14 @@ type DomainConfig struct {
 
 	// Metadata[DomainUniqueName] // .Name + "!" + .Tag
 	// Metadata[DomainTag] // split horizon tag
-	Metadata    map[string]string `json:"meta,omitempty"`
-	Records     Records           `json:"records"`
-	Nameservers []*Nameserver     `json:"nameservers,omitempty"`
+	Metadata         map[string]string `json:"meta,omitempty"`
+	Records          Records           `json:"records"`
+	Nameservers      []*Nameserver     `json:"nameservers,omitempty"`
+	NameserversMutex sync.Mutex        `json:"-"`
 
 	EnsureAbsent Records `json:"recordsabsent,omitempty"` // ENSURE_ABSENT
 	KeepUnknown  bool    `json:"keepunknown,omitempty"`   // NO_PURGE
 
-	IgnoredNames    []*IgnoreName      `json:"ignored_names,omitempty"`
-	IgnoredTargets  []*IgnoreTarget    `json:"ignored_targets,omitempty"`
 	Unmanaged       []*UnmanagedConfig `json:"unmanaged,omitempty"`                      // IGNORE()
 	UnmanagedUnsafe bool               `json:"unmanaged_disable_safety_check,omitempty"` // DISABLE_IGNORE_SAFETY_CHECK
 
@@ -44,6 +44,11 @@ type DomainConfig struct {
 	// 2. Final driver instances are loaded after we load credentials. Any actual provider interaction requires that.
 	RegistrarInstance    *RegistrarInstance     `json:"-"`
 	DNSProviderInstances []*DNSProviderInstance `json:"-"`
+
+	// Pending work to do for each provider.  Provider may be a registrar or DSP.
+	pendingCorrectionsMutex sync.Mutex                 // Protect pendingCorrections*
+	pendingCorrections      map[string]([]*Correction) // Work to be done for each provider
+	pendingCorrectionsOrder []string                   // Call the providers in this order
 }
 
 // GetSplitHorizonNames returns the domain's name, uniquename, and tag.
@@ -86,17 +91,6 @@ func (dc *DomainConfig) Copy() (*DomainConfig, error) {
 	newDc := &DomainConfig{}
 	err := reprint.FromTo(dc, newDc) // Deep copy
 	return newDc, err
-
-	// NB(tlim): The old version of this copied the structure by gob-encoding
-	// and decoding it. gob doesn't like the dc.RegisterInstance or
-	// dc.DNSProviderInstances fields, so we saved a temporary copy of those,
-	// nil'ed out the original, did the gob copy, and then manually copied those
-	// fields using the temp variables we saved. It looked like:
-	//reg, dnsps := dc.RegistrarInstance, dc.DNSProviderInstances
-	//dc.RegistrarInstance, dc.DNSProviderInstances = nil, nil
-	// (perform the copy)
-	//dc.RegistrarInstance, dc.DNSProviderInstances = reg, dnsps
-	//newDc.RegistrarInstance, newDc.DNSProviderInstances = reg, dnsps
 }
 
 // Filter removes all records that don't match the filter f.
@@ -126,7 +120,7 @@ func (dc *DomainConfig) Punycode() error {
 
 		// Set the target:
 		switch rec.Type { // #rtype_variations
-		case "ALIAS", "MX", "NS", "CNAME", "PTR", "SRV", "URL", "URL301", "FRAME", "R53_ALIAS", "NS1_URLFWD", "AKAMAICDN", "CLOUDNS_WR":
+		case "ALIAS", "MX", "NS", "CNAME", "DNAME", "PTR", "SRV", "URL", "URL301", "FRAME", "R53_ALIAS", "NS1_URLFWD", "AKAMAICDN", "CLOUDNS_WR":
 			// These rtypes are hostnames, therefore need to be converted (unlike, for example, an AAAA record)
 			t, err := idna.ToASCII(rec.GetTargetField())
 			if err != nil {
@@ -135,7 +129,7 @@ func (dc *DomainConfig) Punycode() error {
 			rec.SetTarget(t)
 		case "CF_REDIRECT", "CF_TEMP_REDIRECT", "CF_WORKER_ROUTE":
 			rec.SetTarget(rec.GetTargetField())
-		case "A", "AAAA", "CAA", "DS", "LOC", "NAPTR", "SOA", "SSHFP", "TXT", "TLSA", "AZURE_ALIAS":
+		case "A", "AAAA", "CAA", "DHCID", "DNSKEY", "DS", "HTTPS", "LOC", "NAPTR", "SOA", "SSHFP", "SVCB", "TXT", "TLSA", "AZURE_ALIAS":
 			// Nothing to do.
 		default:
 			return fmt.Errorf("Punycode rtype %v unimplemented", rec.Type)
@@ -143,3 +137,39 @@ func (dc *DomainConfig) Punycode() error {
 	}
 	return nil
 }
+
+// StoreCorrections accumulates corrections in a thread-safe way.
+func (dc *DomainConfig) StoreCorrections(providerName string, corrections []*Correction) {
+	dc.pendingCorrectionsMutex.Lock()
+	defer dc.pendingCorrectionsMutex.Unlock()
+
+	if dc.pendingCorrections == nil {
+		// First time storing anything.
+		dc.pendingCorrections = make(map[string]([]*Correction))
+		dc.pendingCorrections[providerName] = corrections
+		dc.pendingCorrectionsOrder = []string{providerName}
+	} else if c, ok := dc.pendingCorrections[providerName]; !ok {
+		// First time key used
+		dc.pendingCorrections[providerName] = corrections
+		dc.pendingCorrectionsOrder = []string{providerName}
+	} else {
+		// Add to existing.
+		dc.pendingCorrections[providerName] = append(c, corrections...)
+		dc.pendingCorrectionsOrder = append(dc.pendingCorrectionsOrder, providerName)
+	}
+}
+
+// GetCorrections returns the accumulated corrections for providerName.
+func (dc *DomainConfig) GetCorrections(providerName string) []*Correction {
+	dc.pendingCorrectionsMutex.Lock()
+	defer dc.pendingCorrectionsMutex.Unlock()
+
+	if dc.pendingCorrections == nil {
+		// First time storing anything.
+		return nil
+	}
+	if c, ok := dc.pendingCorrections[providerName]; ok {
+		return c
+	}
+	return nil
+}
