@@ -328,8 +328,14 @@ var R53_ALIAS = recordBuilder('R53_ALIAS', {
         record.target = args.target;
         if (_.isObject(record.r53_alias)) {
             record.r53_alias['type'] = args.type;
+            if (!_.isString(record.r53_alias['evaluate_target_health'])) {
+                record.r53_alias['evaluate_target_health'] = 'false';
+            }
         } else {
-            record.r53_alias = { type: args.type };
+            record.r53_alias = {
+                type: args.type,
+                evaluate_target_health: 'false',
+            };
         }
     },
 });
@@ -347,6 +353,17 @@ function R53_ZONE(zone_id) {
     };
 }
 
+// R53_EVALUATE_TARGET_HEALTH(enabled)
+function R53_EVALUATE_TARGET_HEALTH(enabled) {
+    return function (r) {
+        if (_.isObject(r.r53_alias)) {
+            r.r53_alias['evaluate_target_health'] = enabled.toString();
+        } else {
+            r.r53_alias = { evaluate_target_health: enabled.toString() };
+        }
+    };
+}
+
 function validateR53AliasType(value) {
     if (!_.isString(value)) {
         return false;
@@ -407,6 +424,47 @@ var DS = recordBuilder('DS', {
     },
 });
 
+// DHCID(name,target, recordModifiers...)
+var DHCID = recordBuilder('DHCID');
+
+// DNAME(name,target, recordModifiers...)
+var DNAME = recordBuilder('DNAME');
+
+// DNSKEY(name, flags, protocol, algorithm, publickey)
+var DNSKEY = recordBuilder('DNSKEY', {
+    args: [
+        ['name', _.isString],
+        ['flags', _.isNumber],
+        ['protocol', _.isNumber],
+        ['algorithm', _.isNumber],
+        ['publickey', _.isString],
+    ],
+    transform: function (record, args, modifiers) {
+        record.name = args.name;
+        record.dnskeyflags = args.flags;
+        record.dnskeyprotocol = args.protocol;
+        record.dnskeyalgorithm = args.algorithm;
+        record.dnskeypublickey = args.publickey;
+        record.target = args.target;
+    },
+});
+
+// name, priority, target, params
+var HTTPS = recordBuilder('HTTPS', {
+    args: [
+        ['name', _.isString],
+        ['priority', _.isNumber],
+        ['target', _.isString],
+        ['params', _.isString],
+    ],
+    transform: function (record, args, modifiers) {
+        record.name = args.name;
+        record.svcpriority = args.priority;
+        record.target = args.target;
+        record.svcparams = args.params;
+    },
+});
+
 // PTR(name,target, recordModifiers...)
 var PTR = recordBuilder('PTR');
 
@@ -488,6 +546,22 @@ var SSHFP = recordBuilder('SSHFP', {
     },
 });
 
+// name, priority, target, params
+var SVCB = recordBuilder('SVCB', {
+    args: [
+        ['name', _.isString],
+        ['priority', _.isNumber],
+        ['target', _.isString],
+        ['params', _.isString],
+    ],
+    transform: function (record, args, modifiers) {
+        record.name = args.name;
+        record.svcpriority = args.priority;
+        record.target = args.target;
+        record.svcparams = args.params;
+    },
+});
+
 // name, usage, selector, matchingtype, certificate
 var TLSA = recordBuilder('TLSA', {
     args: [
@@ -523,11 +597,9 @@ var TXT = recordBuilder('TXT', {
         record.name = args.name;
         // Store the strings from the user verbatim.
         if (_.isString(args.target)) {
-            record.txtstrings = [args.target];
-            record.target = args.target; // Overwritten by the Go code
+            record.target = args.target;
         } else {
-            record.txtstrings = args.target;
-            record.target = args.target.join(''); // Overwritten by the Go code
+            record.target = args.target.join('');
         }
     },
 });
@@ -828,16 +900,6 @@ function DISABLE_IGNORE_SAFETY_CHECK(d) {
     d.unmanaged_disable_safety_check = true;
 }
 
-var IGNORE_NAME_DISABLE_SAFETY_CHECK = {
-    ignore_name_disable_safety_check: 'true',
-    // (NOTE: diff1 only.)
-    // This disables a safety check intended to prevent:
-    // 1. Two owners toggling a record between two settings.
-    // 2. The other owner wiping all records at this label, which won't
-    // be noticed until the next time dnscontrol is run.
-    // See https://github.com/StackExchange/dnscontrol/issues/1106
-};
-
 // IGNORE(labelPattern, rtypePattern, targetPattern)
 function IGNORE(labelPattern, rtypePattern, targetPattern) {
     if (labelPattern === undefined) {
@@ -850,9 +912,6 @@ function IGNORE(labelPattern, rtypePattern, targetPattern) {
         targetPattern = '*';
     }
     return function (d) {
-        // diff1
-        d.ignored_names.push({ pattern: labelPattern, types: rtypePattern });
-        // diff2
         d.unmanaged.push({
             label_pattern: labelPattern,
             rType_pattern: rtypePattern,
@@ -863,30 +922,11 @@ function IGNORE(labelPattern, rtypePattern, targetPattern) {
 
 // IGNORE_NAME(name, rTypes)
 function IGNORE_NAME(name, rTypes) {
-    if (rTypes === undefined) {
-        rTypes = '*';
-    }
-    return function (d) {
-        // diff1
-        d.ignored_names.push({ pattern: name, types: rTypes });
-        // diff2
-        d.unmanaged.push({
-            label_pattern: name,
-            rType_pattern: rTypes,
-        });
-    };
+    return IGNORE(name, rTypes);
 }
 
 function IGNORE_TARGET(target, rType) {
-    return function (d) {
-        // diff1
-        d.ignored_targets.push({ pattern: target, type: rType });
-        // diff2
-        d.unmanaged.push({
-            rType_pattern: rType,
-            target_pattern: target,
-        });
-    };
+    return IGNORE('*', rType, target);
 }
 
 // IMPORT_TRANSFORM(translation_table, domain)
@@ -1463,6 +1503,7 @@ function SPF_BUILDER(value) {
 // iodef_critical: Boolean if sending report is required/critical. If not supported, certificate should be refused. (optional)
 // issue: List of CAs which are allowed to issue certificates for the domain (creates one record for each).
 // issuewild: Allowed CAs which can issue wildcard certificates for this domain. (creates one record for each)
+// ttl: The time for TTL, integer or string. (default: not defined, using DefaultTTL)
 
 function CAA_BUILDER(value) {
     if (!value.label) {
@@ -1482,23 +1523,41 @@ function CAA_BUILDER(value) {
         throw 'CAA_BUILDER requires at least one entry at issue or issuewild';
     }
 
+    var CAA_TTL = function () {};
+    if (value.ttl) {
+        CAA_TTL = TTL(value.ttl);
+    }
     r = []; // The list of records to return.
 
     if (value.iodef) {
         if (value.iodef_critical) {
-            r.push(CAA(value.label, 'iodef', value.iodef, CAA_CRITICAL));
+            r.push(
+                CAA(value.label, 'iodef', value.iodef, CAA_CRITICAL, CAA_TTL)
+            );
         } else {
-            r.push(CAA(value.label, 'iodef', value.iodef));
+            r.push(CAA(value.label, 'iodef', value.iodef, CAA_TTL));
         }
     }
 
-    if (value.issue)
+    if (value.issue) {
+        var flag = function () {};
+        if (value.issue_critical) {
+            flag = CAA_CRITICAL;
+        }
         for (var i = 0, len = value.issue.length; i < len; i++)
-            r.push(CAA(value.label, 'issue', value.issue[i]));
+            r.push(CAA(value.label, 'issue', value.issue[i], flag, CAA_TTL));
+    }
 
-    if (value.issuewild)
+    if (value.issuewild) {
+        var flag = function () {};
+        if (value.issuewild_critical) {
+            flag = CAA_CRITICAL;
+        }
         for (var i = 0, len = value.issuewild.length; i < len; i++)
-            r.push(CAA(value.label, 'issuewild', value.issuewild[i]));
+            r.push(
+                CAA(value.label, 'issuewild', value.issuewild[i], flag, CAA_TTL)
+            );
+    }
 
     return r;
 }
@@ -1697,7 +1756,7 @@ function M365_BUILDER(name, value) {
         );
     }
 
-    r = [];
+    var r = [];
 
     // MX (default: true)
     if (value.mx) {
