@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
+	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
@@ -21,18 +22,10 @@ Info required in `creds.json`:
 // NewDeSec creates the provider.
 func NewDeSec(m map[string]string, metadata json.RawMessage) (providers.DNSServiceProvider, error) {
 	c := &desecProvider{}
-	c.creds.token = m["auth-token"]
-	if c.creds.token == "" {
+	c.token = strings.TrimSpace(m["auth-token"])
+	if c.token == "" {
 		return nil, fmt.Errorf("missing deSEC auth-token")
 	}
-	if err := c.authenticate(); err != nil {
-		return nil, fmt.Errorf("authentication failed")
-	}
-	//DomainIndex is used for corrections (minttl) and domain creation
-	if err := c.initializeDomainIndex(); err != nil {
-		return nil, err
-	}
-
 	return c, nil
 }
 
@@ -41,7 +34,7 @@ var features = providers.DocumentationNotes{
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Can("deSEC always signs all records. When trying to disable, a notice is printed."),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanConcur:              providers.Can(),
 	providers.CanUseAlias:            providers.Unimplemented("Apex aliasing is supported via new SVCB and HTTPS record types. For details, check the deSEC docs."),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Can(),
@@ -119,9 +112,13 @@ func (c *desecProvider) GetZoneRecords(domain string, meta map[string]string) (m
 
 // EnsureZoneExists creates a zone if it does not exist
 func (c *desecProvider) EnsureZoneExists(domain string) error {
-	c.mutex.Lock()
-	defer c.mutex.Unlock()
-	if _, ok := c.domainIndex[domain]; ok {
+	_, ok, err := c.searchDomainIndex(domain)
+	if err != nil {
+		return err
+	}
+
+	if ok {
+		// Domain already exists
 		return nil
 	}
 	return c.createDomain(domain)
@@ -155,14 +152,14 @@ func PrepDesiredRecords(dc *models.DomainConfig, minTTL uint32) {
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
 func (c *desecProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existing models.Records) ([]*models.Correction, error) {
-	var minTTL uint32
-	c.mutex.Lock()
-	if ttl, ok := c.domainIndex[dc.Name]; !ok {
-		minTTL = 3600
-	} else {
-		minTTL = ttl
+	minTTL, ok, err := c.searchDomainIndex(dc.Name)
+	if err != nil {
+		return nil, err
 	}
-	c.mutex.Unlock()
+	if !ok {
+		minTTL = 3600
+	}
+
 	PrepDesiredRecords(dc, minTTL)
 
 	keysToUpdate, toReport, err := diff.NewCompat(dc).ChangedGroups(existing)
@@ -250,9 +247,5 @@ func (c *desecProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, exist
 
 // ListZones return all the zones in the account
 func (c *desecProvider) ListZones() ([]string, error) {
-	var domains []string
-	for domain := range c.domainIndex {
-		domains = append(domains, domain)
-	}
-	return domains, nil
+	return c.listDomainIndex()
 }
