@@ -40,7 +40,17 @@ func nativeToRecords(n gcoreRRSetExtended, zoneName string) ([]*models.RecordCon
 				return nil, fmt.Errorf("unparsable record received from G-Core: %w", err)
 			}
 
-		default: //  "A", "AAAA", "CAA", "NS", "CNAME", "MX", "PTR", "SRV", "TXT"
+		case "TXT": // Avoid double quoting for TXT records
+			if err := rc.SetTargetTXTs(convertSdkAnySliceToTxtSlice(value.Content)); err != nil {
+				return nil, fmt.Errorf("unparsable record received from G-Core: %w", err)
+			}
+
+		case "SCVB": // GCore mistypes "SVCB" as "SCVB"
+			if err := rc.PopulateFromString("SVCB", value.ContentToString(), zoneName); err != nil {
+				return nil, fmt.Errorf("unparsable record received from G-Core: %w", err)
+			}
+
+		default: //  "A", "AAAA", "CAA", "NS", "CNAME", "MX", "PTR", "SRV"
 			if err := rc.PopulateFromString(recType, value.ContentToString(), zoneName); err != nil {
 				return nil, fmt.Errorf("unparsable record received from G-Core: %w", err)
 			}
@@ -79,6 +89,19 @@ func recordsToNative(rcs []*models.RecordConfig, expectedKey models.RecordKey) *
 				Meta:    nil,
 				Enabled: true,
 			}
+		case "TXT": // Avoid double quoting for TXT records
+			rr = dnssdk.ResourceRecord{
+				Content: convertTxtSliceToSdkAnySlice(r.GetTargetTXTJoined()),
+				Meta:    nil,
+				Enabled: true,
+			}
+		case "SVCB":
+			// GCore mistypes "SVCB" as "SCVB"
+			rr = dnssdk.ResourceRecord{
+				Content: dnssdk.ContentFromValue("SCVB", r.GetTargetCombined()),
+				Meta:    nil,
+				Enabled: true,
+			}
 		default:
 			rr = dnssdk.ResourceRecord{
 				Content: dnssdk.ContentFromValue(key.Type, r.GetTargetCombined()),
@@ -107,3 +130,16 @@ func recordsToNative(rcs []*models.RecordConfig, expectedKey models.RecordKey) *
 
 	return result
 }
+
+func convertTxtSliceToSdkAnySlice(record string) []any {
+	result := []any{record}
+	return result
+}
+
+func convertSdkAnySliceToTxtSlice(records []any) []string {
+	result := []string{}
+	for _, record := range records {
+		result = append(result, record.(string))
+	}
+	return result
+}
