@@ -15,21 +15,19 @@ func AuditRecords(records []*models.RecordConfig) []error {
 
 	a.Add("SRV", rejectif.SrvHasNullTarget) // Last verified 20-0212-28
 
+	a.Add("TXT", rejectif.TxtHasBackslash) // Last verified 2023-12-18
+
 	a.Add("TXT", rejectif.TxtHasBackticks) // Last verified 2023-02-02
 
 	a.Add("TXT", rejectif.TxtHasDoubleQuotes) // Last verified 2023-02-02
 
-	a.Add("TXT", rejectif.TxtHasMultipleSegments) // Last verified 2023-02-02
-
-	a.Add("TXT", rejectif.TxtHasSegmentLen256orLonger) // Last verified 2023-02-02
+	a.Add("TXT", rejectif.TxtHasSemicolon) // Last verified 2023-12-18
 
 	a.Add("TXT", rejectif.TxtHasSingleQuotes) // Last verified 2023-02-02
 
 	a.Add("TXT", rejectif.TxtIsEmpty) // Last verified 2023-02-02
 
-	a.Add("TXT", rejectif.TxtIsExactlyLen255) // Last verified 2023-02-02
-
-	a.Add("TXT", rejectif.TxtIsExactlyLen255) // Last verified 2023-02-02
+	a.Add("TXT", rejectif.TxtLongerThan(254)) // Last verified 2023-12-18
 
 	return a.Audit(records)
 }
