@@ -20,7 +20,11 @@ func AuditRecords(records []*models.RecordConfig) []error {
 
 	a.Add("TXT", MaxLengthNDC) // Last verified 2021-03-01
 
-	a.Add("TXT", rejectif.TxtIsEmpty) // Last verified 2021-03-01
+	a.Add("TXT", rejectif.TxtIsEmpty) // Last verified 2023-11-18
+
+	a.Add("TXT", rejectif.TxtHasBackslash) // Last verified 2023-11-18
+	// Backslashes may be allowed in the API but the current
+	// encodeTxt/decodeTxt functions don't support backslashes.
 
 	return a.Audit(records)
 }
@@ -29,18 +33,19 @@ func AuditRecords(records []*models.RecordConfig) []error {
 // are longer than permitted by NDC. Sadly their
 // length limit is undocumented. This seems to work.
 func MaxLengthNDC(rc *models.RecordConfig) error {
-	if len(rc.TxtStrings) == 0 {
+	txtStrings := rc.GetTargetTXTSegmented()
+	if len(txtStrings) == 0 {
 		return nil
 	}
 
 	sum := 2 // Count the start and end quote.
 	// Add the length of each segment.
-	for _, segment := range rc.TxtStrings {
+	for _, segment := range txtStrings {
 		sum += len(segment)                // The length of each segment
 		sum += strings.Count(segment, `"`) // Add 1 for any char to be escaped
 	}
 	// Add 3 (quote space quote) for each interior join.
-	sum += 3 * (len(rc.TxtStrings) - 1)
+	sum += 3 * (len(txtStrings) - 1)
 
 	if sum > 512 {
 		return fmt.Errorf("encoded txt too long")
