@@ -8,7 +8,6 @@ import (
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
-	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
 	"github.com/StackExchange/dnscontrol/v4/providers"
 	"gopkg.in/ns1/ns1-go.v2/rest"
@@ -17,22 +16,30 @@ import (
 )
 
 var docNotes = providers.DocumentationNotes{
+	// The default for unlisted capabilities is 'Cannot'.
+	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Can(),
 	providers.CanGetZones:            providers.Can(),
+	providers.CanConcur:              providers.Can(),
 	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
+	providers.CanUseDNAME:            providers.Can(),
 	providers.CanUseDS:               providers.Can(),
 	providers.CanUseDSForChildren:    providers.Can(),
+	providers.CanUseDHCID:            providers.Can(),
+	providers.CanUseHTTPS:            providers.Can(),
 	providers.CanUseLOC:              providers.Cannot(),
 	providers.CanUseNAPTR:            providers.Can(),
 	providers.CanUsePTR:              providers.Can(),
+	providers.CanUseSVCB:             providers.Can(),
+	providers.CanUseTLSA:             providers.Can(),
 	providers.DocCreateDomains:       providers.Can(),
 	providers.DocDualHost:            providers.Can(),
 	providers.DocOfficiallySupported: providers.Cannot(),
 }
 
-// Number of retries for API backend requests in case of StatusTooManyRequests responses
-const CLIENT_RETRIES = 10
+// clientRetries is the number of retries for API backend requests in case of StatusTooManyRequests responses
+const clientRetries = 10
 
 func init() {
 	fns := providers.DspFuncs{
@@ -69,7 +76,7 @@ func newProvider(creds map[string]string, meta json.RawMessage) (providers.DNSSe
 func (n *nsone) GetZone(domain string) (*dns.Zone, error) {
 	for rtr := 0; ; rtr++ {
 		z, httpResp, err := n.Zones.Get(domain, true)
-		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < CLIENT_RETRIES {
+		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < clientRetries {
 			continue
 		}
 		return z, err
@@ -87,7 +94,7 @@ func (n *nsone) EnsureZoneExists(domain string) error {
 			return nil
 		}
 		// too many requests - retry w/out waiting. We specified rate limit strategy creating the client
-		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < CLIENT_RETRIES {
+		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < clientRetries {
 			continue
 		}
 		return err
@@ -147,7 +154,7 @@ func (n *nsone) GetZoneDNSSEC(domain string) (bool, error) {
 		if err != nil && err == rest.ErrDNSECNotEnabled {
 			return false, nil
 		}
-		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < CLIENT_RETRIES {
+		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < clientRetries {
 			continue
 		}
 		// any other errors not expected, let's surface them
@@ -188,7 +195,7 @@ func (n *nsone) getDomainCorrectionsDNSSEC(domain, toggleDNSSEC string) *models.
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
 func (n *nsone) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
-	corrections := []*models.Correction{}
+	var corrections []*models.Correction
 	domain := dc.Name
 
 	// add DNSSEC-related corrections
@@ -196,49 +203,6 @@ func (n *nsone) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecor
 		corrections = append(corrections, dnssecCorrections)
 	}
 
-	if !diff2.EnableDiff2 {
-
-		existingGrouped := existingRecords.GroupedByKey()
-		desiredGrouped := dc.Records.GroupedByKey()
-
-		differ := diff.New(dc)
-		changedGroups, err := differ.ChangedGroups(existingRecords)
-		if err != nil {
-			return nil, err
-		}
-
-		// each name/type is given to the api as a unit.
-		for k, descs := range changedGroups {
-			key := k
-
-			desc := strings.Join(descs, "\n")
-
-			_, current := existingGrouped[k]
-			recs, wanted := desiredGrouped[k]
-
-			if wanted && !current {
-				// pure addition
-				corrections = append(corrections, &models.Correction{
-					Msg: desc,
-					F:   func() error { return n.add(recs, dc.Name) },
-				})
-			} else if current && !wanted {
-				// pure deletion
-				corrections = append(corrections, &models.Correction{
-					Msg: desc,
-					F:   func() error { return n.remove(key, dc.Name) },
-				})
-			} else {
-				// modification
-				corrections = append(corrections, &models.Correction{
-					Msg: desc,
-					F:   func() error { return n.modify(recs, dc.Name) },
-				})
-			}
-		}
-		return corrections, nil
-	}
-
 	changes, err := diff2.ByRecordSet(existingRecords, dc, nil)
 	if err != nil {
 		return nil, err
@@ -278,7 +242,7 @@ func (n *nsone) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecor
 func (n *nsone) add(recs models.Records, domain string) error {
 	for rtr := 0; ; rtr++ {
 		httpResp, err := n.Records.Create(buildRecord(recs, domain, ""))
-		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < CLIENT_RETRIES {
+		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < clientRetries {
 			continue
 		}
 		return err
@@ -292,7 +256,7 @@ func (n *nsone) remove(key models.RecordKey, domain string) error {
 
 	for rtr := 0; ; rtr++ {
 		httpResp, err := n.Records.Delete(domain, key.NameFQDN, key.Type)
-		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < CLIENT_RETRIES {
+		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < clientRetries {
 			continue
 		}
 		return err
@@ -302,7 +266,7 @@ func (n *nsone) remove(key models.RecordKey, domain string) error {
 func (n *nsone) modify(recs models.Records, domain string) error {
 	for rtr := 0; ; rtr++ {
 		httpResp, err := n.Records.Update(buildRecord(recs, domain, ""))
-		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < CLIENT_RETRIES {
+		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < clientRetries {
 			continue
 		}
 		return err
@@ -325,7 +289,7 @@ func (n *nsone) configureDNSSEC(domain string, enabled bool) error {
 	z.DNSSEC = &enabled
 	for rtr := 0; ; rtr++ {
 		httpResp, err := n.Zones.Update(z)
-		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < CLIENT_RETRIES {
+		if httpResp.StatusCode == http.StatusTooManyRequests && rtr < clientRetries {
 			continue
 		}
 		return err
@@ -346,7 +310,7 @@ func buildRecord(recs models.Records, domain string, id string) *dns.Record {
 		if r.Type == "MX" {
 			rec.AddAnswer(&dns.Answer{Rdata: strings.Fields(fmt.Sprintf("%d %v", r.MxPreference, r.GetTargetField()))})
 		} else if r.Type == "TXT" {
-			rec.AddAnswer(&dns.Answer{Rdata: r.TxtStrings})
+			rec.AddAnswer(&dns.Answer{Rdata: []string{r.GetTargetTXTJoined()}})
 		} else if r.Type == "CAA" {
 			rec.AddAnswer(&dns.Answer{
 				Rdata: []string{
@@ -373,6 +337,17 @@ func buildRecord(recs models.Records, domain string, id string) *dns.Record {
 		} else if r.Type == "NS1_URLFWD" {
 			rec.Type = "URLFWD"
 			rec.AddAnswer(&dns.Answer{Rdata: strings.Fields(r.GetTargetField())})
+		} else if r.Type == "SVCB" || r.Type == "HTTPS" {
+			rec.AddAnswer(&dns.Answer{Rdata: []string{
+				strconv.Itoa(int(r.SvcPriority)),
+				r.GetTargetField(),
+				r.SvcParams}})
+		} else if r.Type == "TLSA" {
+			rec.AddAnswer(&dns.Answer{Rdata: []string{
+				strconv.Itoa(int(r.TlsaUsage)),
+				strconv.Itoa(int(r.TlsaSelector)),
+				strconv.Itoa(int(r.TlsaMatchingType)),
+				r.GetTargetField()}})
 		} else {
 			rec.AddAnswer(&dns.Answer{Rdata: strings.Fields(r.GetTargetField())})
 		}
